#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <vector>
typedef unsigned uint;

// NOTE: Environment variable 'TLIMIT' holds wallclock time you may spend.


//=================================================================================================
// Datatypes:


// ATTRIBUTES/CHARACTERS

struct Attr {
    int     color   : 4;     // -2=undefined, -1=any, 0-7=red, green, blue etc.
    int     size    : 5;     // -2=undefined, 0-9=valid sizes.
    int     scores  : 3;     // 0-3 under scores
    int     bold    : 2;     // -1=either, 0=normal, 1=bold
    int     italic  : 2;     // -1=either, 0=normal, 1=italic
    int     emph    : 3;     // -1=either, 0-2=strength (where 2 is strong emphasis)
    int     tt      : 2;     // 0=normal, 1=tt

    Attr(void) {        // (default constructor defines initial value of parser)
        color = size = -2;
        scores = bold = italic = emph = tt = 0; }

    Attr(const Attr& attr) {
        *this = attr; }

    bool operator == (const Attr& other) const {
        return other.color == color && other.size == size && other.scores == scores && other.bold == bold && other.italic == italic && other.emph == emph && other.tt == tt; }

    bool consistent(const Attr& other) const {
        return (other.color  == -1 || color  == other.color )
            && (other.bold   == -1 || bold   == other.bold  )
            && (other.italic == -1 || italic == other.italic)
            && (other.emph   == -1 || emph   == other.emph  )
            && size   == other.size
            && scores == other.scores
            && tt     == other.tt; }

    void reset(void) {  // The PL operation
        scores = bold = italic = emph = tt = 0; }
};


struct Char : public Attr {
    int     chr;

    Char(int chr_, const Attr& attr) : Attr(attr), chr(chr_) { }
};


// TAGS

enum Tag {
    tag_B , tag_EM, tag_I , tag_PL, tag_S , tag_TT, tag_U , tag_0 , tag_1 , tag_2 , tag_3 , tag_4 , tag_5 ,
    tag_6 , tag_7 , tag_8 , tag_9 , tag_r , tag_g , tag_b , tag_c , tag_m , tag_y , tag_k , tag_w };

const char* on_tag [] = {
    "<B>" , "<EM>", "<I>" , "<PL>", "<S>" , "<TT>", "<U>" , "<0>" , "<1>" , "<2>" , "<3>" , "<4>" , "<5>" ,
    "<6>" , "<7>" , "<8>" , "<9>" , "<r>" , "<g>" , "<b>" , "<c>" , "<m>" , "<y>" , "<k>" , "<w>" };

const char* off_tag [] = {
    "</B>" , "</EM>", "</I>" , "</PL>", "</S>" , "</TT>", "</U>" , "</0>" , "</1>" , "</2>" , "</3>" , "</4>" , "</5>" ,
    "</6>" , "</7>" , "</8>" , "</9>" , "</r>" , "</g>" , "</b>" , "</c>" , "</m>" , "</y>" , "</k>" , "</w>" };


// TOKENS

enum TokenType { tok_ON, tok_OFF, tok_CHAR };
struct Token {
    TokenType   type : 2;
    Tag         tag  : 6;
    char        chr  : 8;
};
inline Token Token_on  (Tag tag) { Token t; t.type = tok_ON;   t.tag = tag; return t; };
inline Token Token_off (Tag tag) { Token t; t.type = tok_OFF;  t.tag = tag; return t; };
inline Token Token_char(int chr) { Token t; t.type = tok_CHAR; t.chr = chr; return t; };

void print(const vector<Token>& out) {
    for (int i = 0; i < (int)out.size(); i++){
        switch (out[i].type){
        case tok_ON  : printf("%s", on_tag [out[i].tag]); break;
        case tok_OFF : printf("%s", off_tag[out[i].tag]); break;
        case tok_CHAR: putchar(out[i].chr);               break;
        default: assert(false); }
    }
}


//=================================================================================================
// Helpers:


int color_letter[] = { 'r', 'g', 'b', 'c', 'm', 'y', 'k', 'w' };


char is_ws[256] = { 0,0,0,0,0,0,0,0,0,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 };
#define isWS(chr) ((bool)is_ws[chr])


//=================================================================================================
// Parsing:


void parse(char* input, vector<Char>& doc)
{
    vector<Attr>    states;
    Attr            state;

    while (*input != '\0') {
        switch (*input){
        case '<' :
            #define PARSE(nchars, action) input += nchars; states.push_back(state); action; break;

            input++;
            switch(*input){
            case 'T': PARSE(3, state.tt     = 1)
            case 'B': PARSE(2, state.bold   = 1)
            case 'I': PARSE(2, state.italic = 1)
            case 'S': PARSE(2, state.emph   = 2)
            case 'E': PARSE(3, if (state.emph < 2) state.emph = 1 - state.emph)
            case 'U': PARSE(2, if (state.scores < 3) state.scores++)
            case 'P': PARSE(3, state.reset())

            case '0': PARSE(2, state.size = 0)
            case '1': PARSE(2, state.size = 1)
            case '2': PARSE(2, state.size = 2)
            case '3': PARSE(2, state.size = 3)
            case '4': PARSE(2, state.size = 4)
            case '5': PARSE(2, state.size = 5)
            case '6': PARSE(2, state.size = 6)
            case '7': PARSE(2, state.size = 7)
            case '8': PARSE(2, state.size = 8)
            case '9': PARSE(2, state.size = 9)

            case 'r': PARSE(2, state.color = 0)
            case 'g': PARSE(2, state.color = 1)
            case 'b': PARSE(2, state.color = 2)
            case 'c': PARSE(2, state.color = 3)
            case 'm': PARSE(2, state.color = 4)
            case 'y': PARSE(2, state.color = 5)
            case 'k': PARSE(2, state.color = 6)
            case 'w': PARSE(2, state.color = 7)

            case '/':
                state = states[states.size()-1];
                states.pop_back();
                do input++; while (*input != '>');
                input++;
                break;

            default: assert(false); }

            break;
            #undef PARSE

        case '>' :  assert(false);

        case ' ' :
        case '\t':
        case '\r':
        case '\n': {
            Char    c(*input, state);
            c.bold = c.italic = c.emph = -1;
            if (state.scores == 0)
                c.color = -1;
            if (state.tt == 0)
                c.chr = ' ';        // <= Replace all WS by SPC (if in non-TT mode)
            doc.push_back(c);
            input++;
            } break;

        default: {
            Char    c(*input, state);
            doc.push_back(c);
            input++;
            }
        }
    }
}


void compressWS(const vector<Char>& in, vector<Char>& out)
{
    if (in.size() == 0) return;

    out.push_back(in[0]);
    for (unsigned i = 1; i < in.size(); i++){
        if (isWS(in[i].chr)){
            if (in[i].tt == 1)
                out.push_back(in[i]);
            else if (out[out.size()-1].chr == ' ' && (Attr)in[i] == (Attr)in[i-1])
//          else if (in[i-1].chr == ' ' && (Attr)in[i] == (Attr)in[i-1])    // (first specification...)
                /*nothing*/;
            else
                out.push_back(Char(' ', Attr(in[i])));
        }else
            out.push_back(in[i]);
    }
}


char* readStdin(void)
{
    char*   buf      = (char*)malloc(1);
    int     capacity = 1;
    int     size     = 0;
    int     chr;
    while (chr = getchar(),
           chr != EOF){
        if (size >= capacity){ capacity *= 2; buf = (char*)realloc(buf, capacity); }
        buf[size++] = chr;
    }
    if (size >= capacity){ capacity *= 2; buf = (char*)realloc(buf, capacity); }
    buf[size++] = '\0';
    return buf;
}


//=================================================================================================
// Optimizer:


#define MAX_POPS 10


inline bool haveToReset(Attr from, Attr to) {
    return ( (from.tt     == 1 && to.tt         == 0)
          || (from.italic == 1 && to.italic     == 0)
          || (from.bold   == 1 && to.bold       == 0)
          || (from.emph   == 2 && (to.emph & 2) == 0)
          || (from.scores > to.scores)); }


inline int cost(Attr from, Attr to)
{
    #define color_cost  7
    #define size_cost   7
    #define U_cost      7
    #define B_cost      7
    #define I_cost      7
    #define S_cost      7
    #define EM_cost     9
    #define TT_cost     9
    #define PL_cost     9

    int     cost;

    if (haveToReset(from, to))
        from.reset(),
        cost = PL_cost;
    else
        cost = 0;

    if (to.color  != -1 && from.color  != to.color ) cost += color_cost;
    if (                   from.size   != to.size  ) cost += size_cost;
    if (                   from.scores != to.scores) cost += U_cost * (to.scores - from.scores);
    if (to.bold   != -1 && from.bold   != to.bold  ) cost += B_cost;
    if (to.italic != -1 && from.italic != to.italic) cost += I_cost;
    if (to.emph   != -1 && from.emph   != to.emph  ) cost += (to.emph == 2) ? S_cost : EM_cost;
    if (                   from.tt     != to.tt    ) cost += TT_cost;

    return cost;
}


void perform(Attr& state, Attr to, vector<Tag>& tag_stack, vector<Attr>& state_stack, vector<Token>& out)
{
    #define PUSH(tag) (tag_stack.push_back(tag), state_stack.push_back(state), out.push_back(Token_on(tag)))

    if (haveToReset(state, to))
        PUSH(tag_PL),
        state.reset();

    if (to.color != -1 && state.color != to.color){
        PUSH((Tag)((int)tag_r + to.color));
        state.color = to.color; }

    if (state.size != to.size){
        PUSH((Tag)((int)tag_0 + to.size));
        state.size = to.size; }

    if (state.scores != to.scores)
        for (int i = 0; i < (to.scores - state.scores); i++)
            PUSH(tag_U),
            state.scores++;

    if (to.bold != -1 && state.bold != to.bold)
        PUSH(tag_B),
        state.bold = 1;

    if (to.italic != -1 && state.italic != to.italic)
        PUSH(tag_I),
        state.italic = 1;

    if (to.emph != -1 && state.emph != to.emph){
        if (to.emph == 2)
            PUSH(tag_S);
        else
            PUSH(tag_EM);
        state.emph = to.emph; }

    if (state.tt != to.tt)
        PUSH(tag_TT),
        state.tt = 1;

    #undef PUSH
}


void perform_forcedPops(Attr& state, Attr to, vector<Tag>& tag_stack, vector<Attr>& state_stack, vector<Token>& out)
{
    if (to.color == -2){
        while (state.color != -2){
            out.push_back(Token_off(tag_stack.back()));
            tag_stack.pop_back();
            state = state_stack.back();
            state_stack.pop_back(); }
    }

    if (to.size == -2){
        while (state.size != -2){
            out.push_back(Token_off(tag_stack.back()));
            tag_stack.pop_back();
            state = state_stack.back();
            state_stack.pop_back(); }
    }
}


void maybePop(Attr& state, Attr to, vector<Tag>& tag_stack, vector<Attr>& state_stack, vector<Token>& out)
{
    if (state == to || state_stack.size() == 0) return;

    int     best;
    int     best_n;
    int     i, n;
    int     d;

    best = cost(state, to);
    best_n = 0;
    for (i = (int)state_stack.size()-1, n = 1; i >= 0 && n <= MAX_POPS; i--, n++){
        d = cost(state_stack[i], to);
        if (d < best)
            best = d, best_n = n;
    }

    for (i = 0; i < best_n; i++){
        out.push_back(Token_off(tag_stack.back()));
        tag_stack.pop_back();
        state = state_stack.back();
        state_stack.pop_back();
    }
}


void greedy(const vector<Char>& doc, vector<Token>& out)
{
    Attr            state;
    vector<Tag>     tags;
    vector<Attr>    states;
    int             i;

    for (i = 0; i < (int)doc.size(); i++){
        perform_forcedPops(state, doc[i], tags, states, out);
        maybePop          (state, doc[i], tags, states, out);
        perform           (state, doc[i], tags, states, out);
        out.push_back(Token_char(doc[i].chr));
    }

    for (int j = (int)tags.size()-1; j >= 0; j--)
        out.push_back(Token_off(tags[j]));
}


int main(void)
{
    vector<Char>    doc, doc2;
    vector<Token>   out;

    char* tmp = readStdin();
    parse(tmp, doc);
    free(tmp);

    compressWS(doc, doc2);
    greedy(doc2, out);
    print(out);

    return 0;
}
